'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

let _stringifySqlAST = (() => {
  var _ref2 = _asyncToGenerator(function* (parent, node, prefix, context, selections, joins, wheres) {
    switch (node.type) {
      case 'table':
        if (node.where) {
          const whereCondition = yield node.where(`"${node.as}"`, node.args || {}, context, prefix);
          if (whereCondition) {
            wheres.push(`${whereCondition}`);
          }
        }

        if (node.sqlJoin) {
          const joinCondition = yield node.sqlJoin(`"${parent.as}"`, `"${node.as}"`, node.args || {}, context);

          joins.push(`LEFT JOIN ${node.name} AS "${node.as}" ON ${joinCondition}`);
        } else if (node.joinTable) {
          if (!node.sqlJoins) throw new Error('Must set "sqlJoins" for a join table.');
          const joinCondition1 = yield node.sqlJoins[0](`"${parent.as}"`, `"${node.joinTableAs}"`, node.args || {}, context);
          const joinCondition2 = yield node.sqlJoins[1](`"${node.joinTableAs}"`, `"${node.as}"`, node.args || {}, context);

          joins.push(`LEFT JOIN ${node.joinTable} AS "${node.joinTableAs}" ON ${joinCondition1}`, `LEFT JOIN ${node.name} AS "${node.as}" ON ${joinCondition2}`);
        } else {
          joins.push(`FROM ${node.name} AS "${node.as}"`);
        }

        for (let child of node.children) {
          yield _stringifySqlAST(node, child, [...prefix, node.as], context, selections, joins, wheres);
        }

        break;
      case 'column':
        selections.push(`"${parent.as}"."${node.name}" AS "${(0, _shared.joinPrefix)(prefix) + node.as}"`);
        break;
      case 'columnDeps':
        for (let name in node.names) {
          selections.push(`"${parent.as}"."${name}" AS "${(0, _shared.joinPrefix)(prefix) + node.names[name]}"`);
        }
        break;
      case 'composite':
        const keys = node.name.map(function (key) {
          return `"${parent.as}"."${key}"`;
        });

        selections.push(`${keys.join(' || ')} AS "${(0, _shared.joinPrefix)(prefix) + node.fieldName}"`);
        break;
      case 'expression':
        const expr = node.sqlExpr(`"${parent.as}"`, node.args || {}, context);
        selections.push(`${expr} AS "${(0, _shared.joinPrefix)(prefix) + node.as}"`);
        break;
      case 'noop':
        return;
      default:
        throw new Error('unexpected/unknown node type reached: ' + (0, _util.inspect)(node));
    }
    return { selections, joins, wheres };
  });

  return function _stringifySqlAST(_x3, _x4, _x5, _x6, _x7, _x8, _x9) {
    return _ref2.apply(this, arguments);
  };
})();

var _util = require('../util');

var _shared = require('./shared');

function _asyncToGenerator(fn) { return function () { var gen = fn.apply(this, arguments); return new Promise(function (resolve, reject) { function step(key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { return Promise.resolve(value).then(function (value) { step("next", value); }, function (err) { step("throw", err); }); } } return step("next"); }); }; }

exports.default = (() => {
  var _ref = _asyncToGenerator(function* (topNode, context) {
    (0, _util.validateSqlAST)(topNode);

    let { selections, joins, wheres } = yield _stringifySqlAST(null, topNode, [], context, [], [], []);

    selections = [...new Set(selections)];

    if (!selections.length) return '';

    let sql = 'SELECT\n  ' + selections.join(',\n  ') + '\n' + joins.join('\n');
    if (wheres.length) {
      sql += '\nWHERE ' + wheres.join(' AND ');
    }
    return sql;
  });

  function stringifySqlAST(_x, _x2) {
    return _ref.apply(this, arguments);
  }

  return stringifySqlAST;
})();